// Import required libraries
#include <Wire.h>
#include "ESP8266WiFi.h"
#include <PubSubClient.h>

#include <DHT.h> // DHT has no pressure sensor

// WiFi parameters
const char* ssid = "Athena";
const char* password = "AthenaRowledge";
const char* mqtt_server = "Amtiskaw";

WiFiClient espClient;
PubSubClient client(espClient);

// Variables to be exposed to the API
float temperature;
//float pressure;
float humidity;
char valBuff[7];
int chipID;
char tempTopic[150], pressureTopic[150], humidityTopic[150], statusTopic[150];

// Initialize DHT sensor
// DHT11 sensor pins
#define DHTPIN 5
#define DHTTYPE DHT22
#define sensor dht
DHT dht(DHTPIN, DHTTYPE, 15);

void connect() {
 // Loop until we're reconnected
 while (!client.connected()) {
 Serial.print("Attempting MQTT connection...");
 // Attempt to connect
 if (client.connect("ESP8266 Client")) {
  Serial.println("connected");
  client.publish(statusTopic, "DHT22 ok");
 } else {
  Serial.print("failed, rc=");
  Serial.print(client.state());
  Serial.println(" try again in 5 seconds");
  // Wait 5 seconds before retrying
  delay(5000);
  }
 }
}

void setup(void)
{  
  // Start Serial
  Serial.begin(115200);
  chipID = ESP.getChipId();

  snprintf(tempTopic, 149, "weatherstation/indoortemp/%08X", chipID);
//  snprintf(pressureTopic, 149, "weatherstation/indoorpressure/%08X", chipID);
  snprintf(humidityTopic, 149, "weatherstation/indoorhumidity/%08X", chipID);
  snprintf(statusTopic, 149, "weatherstation/status/%08X", chipID);
  
  Serial.printf("ESP8266 serial: %08X\n", chipID);
  
//  if (!sensor.begin()) {  
  sensor.begin(); // DHT lib doesn't return a bool error code 
    Serial.println("Could not find a valid SHT31 sensor, check wiring!");
//    while (1);
//  }
    
  client.setServer(mqtt_server, 1883);
  connect();

  // Reading temperature and humidity
  temperature = sensor.readTemperature();
//  pressure = sensor.readPressure();
  humidity = sensor.readHumidity();

  if ( ! isnan(temperature)) {
    String(temperature, 1).toCharArray(valBuff, 7);
    client.publish(tempTopic, valBuff);
  }
//  if ( ! isnan(pressure)) {
//    String(pressure, 1).toCharArray(valBuff, 7);
//    client.publish(pressureTopic, valBuff);
//  }
  if ( ! isnan(humidity)) {
    String(humidity, 1).toCharArray(valBuff, 7);
    client.publish(humidityTopic, valBuff);
  }
 
}

void loop() {
 // now sleep for a while, using minimal power
  ESP.deepSleep(10 * 60 * 1000000);

}
